@@ -31,20 +31,8 @@
 extern "C" {
 #endif
 
-#define LED1          BSP_IO_PORT_01_PIN_11 // D13
-#define LED_STATE_ON  1
-
-#define SW1                   BSP_IO_PORT_01_PIN_10 // D12
-#define BUTTON_STATE_ACTIVE   0
-
-static const ioport_pin_cfg_t board_pin_cfg[] = {
-    {.pin = LED1, .pin_cfg = IOPORT_CFG_PORT_DIRECTION_OUTPUT},
-    {.pin = SW1, .pin_cfg = IOPORT_CFG_PORT_DIRECTION_INPUT},
-    // USB FS D+, D-, VBus
-    {.pin = BSP_IO_PORT_04_PIN_07, .pin_cfg = IOPORT_CFG_PERIPHERAL_PIN | IOPORT_PERIPHERAL_USB_FS},
-    {.pin = BSP_IO_PORT_09_PIN_14, .pin_cfg = IOPORT_CFG_PERIPHERAL_PIN | IOPORT_PERIPHERAL_USB_FS},
-    {.pin = BSP_IO_PORT_09_PIN_15, .pin_cfg = IOPORT_CFG_PERIPHERAL_PIN | IOPORT_PERIPHERAL_USB_FS},
-};
+#define LED_STATE_ON         1
+#define BUTTON_STATE_ACTIVE  0
 
 #ifdef __cplusplus
 }
